(************************************************************************)
(* hashmap-sig.sml                                                      *)
(*                                                                      *)
(* Description:                                                         *)
(*    An interface for imperative dictionary structures (e.g. hash      *)
(*    tables).                                                          *)
(*                                                                      *)
(* Author:                                                              *)
(*    Evan Chang                                                        *)
(************************************************************************)

(* $Id: *)

signature HASHMAP =
sig

  type key
  type 'a entry = key * 'a

  type 'a table

  (******************************************)
  (* Create a new table given a size hint   *)
  (******************************************)
  val new : int -> 'a table


  (******************************************)
  (* Insert into the table                  *)
  (******************************************)

  (* If the key already has an associated entry, then it is removed. *)
  val insert : 'a table -> 'a entry -> unit


  (******************************************)
  (* Lookup in the table                    *)
  (******************************************)
  val lookup : 'a table -> key -> 'a option


  (******************************************)
  (* Clear the table                        *)
  (******************************************)
  val clear : 'a table -> unit

end; (* signature HASHMAP *)
