(************************************************************************)
(* hashmapr-sig.sml                                                     *)
(*                                                                      *)
(* Description:                                                         *)
(*    A more rich interface for imperative dictionary structures        *)
(*    (e.g. hash tables).  See hashmap-sig.sml for a simpler interface. *)
(*                                                                      *)
(* Author:                                                              *)
(*    Evan Chang                                                        *)
(************************************************************************)

(* $Id: *)

signature HASHMAPR =
sig

  type key
  type 'a entry = key * 'a

  type 'a table

  (******************************************)
  (* Create a new table given a size hint   *)
  (******************************************)
  val new : int -> 'a table


  (******************************************)
  (* Insert into the table                  *)
  (******************************************)

  (* If the key already has an associated entry, then it is removed. *)
  val insert : 'a table -> 'a entry -> unit

  (* Returns an entry that is already associated with the given key if
     one exists *)
  val insertr : 'a table -> 'a entry -> 'a entry option


  (******************************************)
  (* Lookup in the table                    *)
  (******************************************)

  (* Locate the entry index by the given key *)
  val lookup : 'a table -> key -> 'a option


  (******************************************)
  (* Remove entries                         *)
  (******************************************)

  (* Returns the removed entry if it exists *)
  val remove : 'a table -> key -> 'a entry option


  (******************************************)
  (* Clear the table                        *)
  (******************************************)
  val clear : 'a table -> unit


  (******************************************)
  (* Output List                            *)
  (******************************************)
  val toList  : 'a table -> 'a list
  val toListi : 'a table -> 'a entry list


  (******************************************)
  (* Iteration                              *)
  (******************************************)
  (* maybe add some iteration functions: app, map, etc. *)

end; (* signature HASHMAPR *)
