(************************************************************************)
(* headcheck-sig.sml                                                    *)
(*                                                                      *)
(* Description:                                                         *)
(*     Provides an inferface for checking if a formula is worth         *)
(*     focusing on.                                                     *)
(*                                                                      *)
(* Author:                                                              *)
(*     Evan Chang                                                       *)
(************************************************************************)

(* $Id: headcheck-sig.sml,v 1.2 2002/06/21 04:12:32 bechang_conductor Exp $ *)


(************************************************************************)
(* signature HEADCHECK                                                  *)
(*                                                                      *)
(*   Interface:                                                         *)
(*                                                                      *)
(*     This signature provides a check for determining if a formula     *)
(*     should be focused on as an optimization.                         *)
(*                                                                      *)
(************************************************************************)
signature HEADCHECK =
sig

  (* Linear Logic Propositions *)
  structure P : PROP

  (* Focusing Structure *)
  structure F : FOCUSING

  (* Goals - parametric in P *)
  datatype goal =                       (* G ::=           *)
    GUnify of P.prop                    (*     | P' = P    *)
  | GTensor of goal * P.prop            (*     | G (x) A   *)
  | GOr of goal * goal                  (*     | G1 (+) G2 *)
  | GZero                               (*     | 0         *)
  | GTensorBang of goal * P.prop        (*     | G (x)! A  *)
  | GLolli of P.prop                    (*     | A -o P    *)


  (* headcheck dbglog (G,D,A,P) = gs

     Given possible left focus formula A and atomic goal P.  Determine
     whether A possibly entails P.  If A possibly entails P, a list of
     remaining goals to be shown is returned.
  *)
  val headcheck : (string -> unit)
                    -> F.C.unrestrictedCxt * F.C.linearCxt * P.prop * P.prop
                    -> F.C.constraints 
                    -> (F.C.constraints * F.sequent list) option

end; (* signature HEADCHECK *)
