(************************************************************************)
(* obdd-sig.sml                                                         *)
(*                                                                      *)
(* Description:                                                         *)
(*    An interface for ordered binary decision diagrams (OBDDs).        *)
(*                                                                      *)
(* Author:                                                              *)
(*    Evan Chang                                                        *)
(************************************************************************)

(* $Id: *)

signature OBDD =
sig

  (******************************************)
  (* Boolean variables                      *)
  (******************************************)
  type var = int


  (******************************************)
  (* Binary boolean operators               *)
  (******************************************)
  datatype oper =
    And
  | Or
  | Imp
  | BiImp
  | Xor
  | Nand
  | Nor


  (******************************************)
  (* OBDD                                   *)
  (******************************************)
  type obdd

  val zero      : obdd
  val one       : obdd

  val mkVar     : var -> obdd

  val not       : obdd -> obdd
  val apply     : oper -> obdd * obdd -> obdd

  val eq        : obdd * obdd -> bool
  val toString  : obdd -> string

end; (* signature OBDD *)
