(************************************************************************)
(* uniqueidsrc-cml.sml                                                  *)
(*                                                                      *)
(* Description:                                                         *)
(*    An implementation for a unique ID source thread-safe for CML.     *)
(*                                                                      *)
(* Author:                                                              *)
(*    Evan Chang                                                        *)
(************************************************************************)

(* $Id: *)


(**************************************************************)
(* functor UniqueIdSrcCML                                     *)
(*                                                            *)
(* Create a unique id source.                                 *)
(**************************************************************)
functor UniqueIdSrcCML(I : UNIQUEID) :> UNIQUEIDSRC where type id = I.id =
struct

  type id = I.id
  type idsource = unit -> id

  (******************************************)
  (* Create a new source.                   *)
  (******************************************)
  fun new () =
      let
        open CML
        val idchan = channel ()

        fun loop nextid =
            let
              val _ = send (idchan, nextid)
            in
              loop (I.next nextid)
            end

        (* Start server loop *)
        val _ = spawn (fn () => loop I.initial)
      in
        fn () => recv idchan
      end

end; (* functor UniqueIdSrcCML *)
